#include <curl/curl.h>
#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include "opt_exists.c"

bool start_unit_output = false;

char *skip_file = NULL;

int start_unit(const Curl_fileinfo *finfo, void *ptr, int remains)
{
  if (start_unit_output) {
    fprintf(stderr, "======================================================\n");
    fprintf(stderr, "==== STARTUNIT %s", finfo->filename);
    if (finfo->filetype == CURLFILETYPE_DIRECTORY)
      fprintf(stderr, " (directory)\n");
    else
      fprintf(stderr, "\n");
  }
  if (skip_file) {
    if (strcmp(skip_file, finfo->filename) == 0) {
      return 1;
    }
  }

  return 0;
}

int end_unit(void *ptr)
{
  if (start_unit_output) {
    fprintf(stderr, "==== ENDUNIT \n");
    fprintf(stderr, "======================================================\n");
  }
  return 0;
}

size_t write_data(void *buffer, size_t size, size_t nmemb, void *userp)
{
  fwrite(buffer, size, nmemb, userp);
  return size*nmemb;
}

int main(int argc, char **argv)
{
  curl_global_init(CURL_GLOBAL_ALL);
  CURL *curl = curl_easy_init();
  char *optreturn;
  char url[1000] = ""; /* probably enough */
  FILE *f = NULL;
  char errorbuffer[CURL_ERROR_SIZE];

  if((optreturn = opt_exists(argc, argv, "-help"))  != NULL ||
     (optreturn = opt_exists(argc, argv, "-h"))  != NULL ||
     (optreturn = opt_exists(argc, argv, "--help")) != NULL) {
    printf(
      " -help\n"
      " -verbose                (enables curl verbose output)\n"
      " -show_url               (shows set url)\n"
      " -domain=ftp://user:pass@domain.tld\n"
      " -uri=/*.txt\n"
      " -null                   (output is set to /dev/null)\n"
      " -repeat=N               (repeats download the same target N times)\n"
      " -show_rc                (after each curl_easy_perform() prints\n"
      "                          RETURN CODE of call)\n"
      " -wc                     (turns on wildcard download)\n"
      " -SUEU                   (enables startunit/endunit callbacks)\n"
      " -skip=FILENAME          (skip file downloading in startunit callback)\n"
    );
    return 0;
  }

  if((optreturn = opt_exists(argc, argv, "-domain=")) != NULL) {
    strcat(url, optreturn);
  }
  else {
    strcat(url, "ftp://curltest.howto.cz:123456@curltest.howto.cz");
  }

  if(opt_exists(argc, argv, "-SUEU")) {
    start_unit_output = true;
  }

  curl_easy_setopt(curl, CURLOPT_ENDUNIT, end_unit);
  curl_easy_setopt(curl, CURLOPT_STARTUNIT, start_unit);
  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);

  if((optreturn = opt_exists(argc, argv, "-uri=")) != NULL) {
    strcat(url, optreturn);
  }

  if(opt_exists(argc, argv, "-verbose")) {
    curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
  }

  if((optreturn = opt_exists(argc, argv, "-skip=")) != NULL) {
    skip_file = optreturn;
  }

  if(opt_exists(argc, argv, "-wc")) {
    curl_easy_setopt(curl, CURLOPT_WILDCARDMATCH, 1L);
  }

  curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorbuffer);
  curl_easy_setopt(curl, CURLOPT_URL, url);

  if(opt_exists(argc, argv, "-show_url")) {
    fprintf(stderr, ">> %s <<\n", url);
  }

  if(opt_exists(argc, argv, "-null")) {
    f = fopen("/dev/null", "w");
    if (!f) {
      fprintf(stderr, "cannot open /dev/null\n");
      return 1;
    }
    else
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, f);
  }



  int loop = 1;
  optreturn = opt_exists(argc, argv, "-repeat=");
  if (optreturn) {
    loop = atol(optreturn);
    if (loop <= 0) {
      loop = 1;
    }
  }
  for (int i = 0; i < loop; i++) {
    int ret = curl_easy_perform(curl);
    if (opt_exists(argc, argv, "-show_rc")) {
      if (ret)
        fprintf(stderr, "RETURN CODE = %d (%s)\n", ret, errorbuffer);
      else
        fprintf(stderr, "RETURN CODE = CURLE_OK\n");
    }
  }

  if (f)
    fclose(f);
  curl_easy_cleanup(curl);
  curl_global_cleanup();
}
